;******************************************************
; ProcChr - decode host cmd and update disp
;
; Designed and written by Daryl Rictor (c)2003-2004
;
;
;  Modified by Grant Searle 2007
;******************************************************
ProcChr:

; If a command is waiting for a parameter, use the "inpt" value as the parameter
; for the command. The ID of the command is held in cmdVal
	mov	J,cmdVal
	cpi	J, 0x00				; Check if a command is waiting for a parameter
	brne	PROCESS_COMMAND			; If so, process thay command

	tst	fnthl				; is font set to lower?
	breq	proc1				; yes, use the jmp table
	cpi	inpt, 0x80			; no, is it a lower 128 code?
	brcc	proc1				; yes, just treat as standard char
	jmp	ASCII
proc1:
	ldi 	ZH, 0x05			; no, setup jump table page address
	mov	ZL, inpt			; put the input chr into Z
	ijmp					; jump table based on INPT

;========================================================
; Commands waiting for a parameter
;========================================================

PROCESS_COMMAND:
cmd1:
	mov	J,cmdVal
	cpi	J, 0xC0				; set column
	brne	cmd2
	cpi	inpt, 0x50			; check column = 0-79
	brlo	SET_COL
	rjmp	cmdComplete
SET_COL:
	sub	YL, hpos			;
	sbci	YH, 0x00			; 
	mov	hpos, inpt			; update hpos
	ldi	inpt, 0x00			; 
	add	YL, hpos			;
	adc	YH, inpt			;
	ld	curschr, Y			; save chr under cursor
	rjmp	cmdComplete

cmd2:
	mov	J,cmdVal
	cpi	J, 0xC1				; set row
	brne	cmd3
	cpi	inpt, 0x19			; check row = 0-24
	brlo	SET_ROW
	rjmp	cmdComplete
SET_ROW:
	ldi	YL, 0x50			; # chrs per line
	ldi	YH, 0x00			; offset to start of SRAM 60
	andi	inpt, 0x1F			; mask down to Line 0-24
	mov	vpos, inpt			; save new vertical position
	mul	YL, vpos			; multiply by chrs per line
	ldi	inpt, 0x01			; need a zero for upper byte addition 00
	add	R0, YH				; add SRAM start offset to result
	adc	R1, inpt			; add the carry bit
	movw	YL, R0				; mov it back to Y reg
	ldi	inpt, 0x00			; 2013.3.4
	add	YL, hpos			; add in hpos
	adc	YH, inpt			; add the carry bit
	ld	curschr, Y			; save chr under cursor
	rjmp	cmdComplete

cmd3:
cmdComplete:
	ldi	J,0x00
	mov	cmdVal, J
	ret

;========================================================
; Commands called from the Jump Table
;========================================================

SET_PRIM:
	ldi	inpt, 0x00
	mov	fntsel, inpt			; select primary font
	ret

SET_ALT:
	ldi	inpt, 0x08
	mov	fntsel, inpt			; select alternate font
	ret

SET_UPPER:
	ldi	inpt, 0x80
	mov	fnthl, inpt			; select upper 128 bytes of the font
	ret

SET_LOWER:
	ldi	inpt, 0x00
	mov	fnthl, inpt			; select lower 128 bytes of the font
	ret

TAB:
	ldi	inpt, 0x20			; convert tabs to white spaces
	st	Y+, inpt			; save space & advance
	inc	hpos				; 
	ldi	inpt, 0x07			; tabs are fixed  0,8,16,24,32, 40 (will wrap to next line if=40)	
	and	inpt, hpos			; mask out cursor column
	brne	TAB				; not to next tab, so do again
	rjmp	wrap				; did it wrap?

DEL_EOL:
	ldi	inpt, 0x20			; 
	push	YL
	push	YH
	push	hpos
deleol1:
	st	Y+, inpt			; 
	inc	hpos				; 
	cpi	hpos, 0x50			;
	brne	deleol1				; clear to end of line
	pop	hpos
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

DEL_SOL:
	ldi	inpt, 0x20			; 
	push	YL
	push	YH
	push	hpos
	st	Y,inpt
delsol1:
	st	-Y, inpt			; 
	dec	hpos				; 
	cpi	hpos, 0x00			;
	brne	delsol1				; clear to start of line
	pop	hpos
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

DEL_SOP:					; del to start of page
	ldi	inpt, 0x20			; 
	push	YL
	push	YH
	st	Y,inpt
delsop1:
	st	-Y, inpt			;
	cpi	YH, 0x01			; 00
	brne	delsop1				;
	cpi	YL, 0x00			; 60
	brne	delsop1
	pop	YH
	pop	YL
	ld	curschr, Y			;
	ret

DEL_EOP:					; del to end of page
	ldi	inpt, 0x20			; 
	push	YL
	push	YH
	st	Y,inpt
deleop1:
	st	Y+, inpt			; 
	cpi	YH, 0x08			; 08
	brne	deleop1				;
	cpi	YL, 0xD0			; 30
	brne	deleop1
	pop	YH
	pop	YL
	ld	curschr, Y			;
	ret

UNKNOWN:
	ldi	inpt, 0x3F  ; "?"
	rjmp	ascii

SPECIAL:
	andi	inpt, 0x1F			; mask out 0-31, and fall thru to ASCII code
ASCII:
	or	inpt, fnthl			; mask in the lower/upper select bit
	st	Y+, inpt			; store chr
	inc	hpos				; inc hpos
WRAP:
	cpi	hpos, 0x50			; past eol?
	brne	wrap1				; yes, continue
	ldi	hpos, 0x00			; reset hpos
	inc	vpos				; inc v pos
	cpi	vpos, 0x19			; is it below last line (25)?
	brne	wrap1				; no, do cursor and exit
	sbiw	YL, 0x20			; adjust Y to first char
	sbiw	YL, 0x30			; adjust Y to first char
	ldi	vpos, 0x18			; set to last line (24)
	rjmp	scrollup	
wrap1:
	ld	curschr, Y			; save chr under cursor
	ret					; return to Main

RETURN: 
	sub	YL, hpos			;
	sbci	YH, 0x00			; 
	ldi	hpos, 0x00			; update hpos
	ld	curschr, Y			; save chr under cursor
	ret					; return to Main

LINEFEED:
	cpi	vpos, 0x18			; are we at line 24?
	breq	SCROLLUP			; yes, need to scroll up one line
	inc	vpos				; no, move cursor down 1 line
	adiw	YL, 0x20			; adjust Y pointer
	adiw	YL, 0x30			; adjust Y pointer
	ld	curschr, Y			; save chr under cursor
	ret					; return to Main

DELETE:
	ldi	inpt, 0x20			; space
	st	Y, inpt				; save to SRAM
	mov	Curschr, inpt			; save chr under cursor (space)
	ret					; return to Main

BACKSPACE:
	cpi	hpos, 0x00			; are we already at the left edge?
	brne	del1				; no, move left one chr
	cpi	vpos, 0x00			; are we already on the top line?
	breq	del2  				; yes, nothing to do, so end
	dec	vpos				; no, move up one line
	ldi	hpos, 0x50			; & set the cursor to the right-most chr (+1 for next instruction)
del1:	
	dec	hpos				; move cursor left one chr
	ldi	inpt, 0x20			; space
	st	-Y, inpt			; remove last chr (destructive BS)
	mov	Curschr, inpt			; save chr under cursor (space)
del2:
	ret					; return to Main

CURSHOME:					; move cursor to Home w/o clear
	ldi	YH, 0x00			; 
	rjmp	formfeed1

FORMFEED:					; Clear the Screen
	ldi	ZL, 0x00
	ldi	ZH, 0x01			; set to start of SRAM
	ldi	YL, 0xD0
	ldi	YH, 0x07			; set Y to 2000
	ldi	inpt, 0x20			; " " space chracter
ffloop:
	st 	Z+, inpt			; save to SRAM
	sbiw	YL, 0x01			; dec Y
	brne	ffloop				; do until Y=0
formfeed1:
	ldi	YL, 0x00			; set Y to first chr 60
	ldi	YH, 0x01			;
	ldi	hpos, 0x00			; set cursor to top left pos
	ldi	vpos, 0x00			; 
	ld	curschr, Y			; save chr under cursor
	ret					; return to Main

SCROLLUP:
	push	YL
	push	YH
	ldi	YL, 0x00			; start of first line 60
	ldi	YH, 0x01			; 00
	ldi	ZL, 0x50			; start of second line B0
	ldi	ZH, 0x01			; 00
scup1:
	ld	inpt, Z+			; get line 2 
	st	Y+, inpt			; place in line 1
	cpi	ZL, 0xD0			; 30
	brne	scup1
	cpi	ZH, 0x08			; at the end of disp RAM?
	brne	scup1				; no, do again
	ldi	inpt, 0x20			; space
scup2:
	st	Y+, inpt			; fill last line with spaces
	cpi	YL, 0xD0			; 30
	brne	scup2
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

SCROLLDN:
	push	YL
	push	YH
	ldi	YL, 0x80			; end of line 24  E0
	ldi	YH, 0x08			; 07
	ldi	ZL, 0xD0			; end of line 25  30
	ldi	ZH, 0x08			; 
scdn1:
	ld	inpt, -Y			; get line 2 
	st	-Z, inpt			; place in line 1
	cpi	YL, 0x00			; 60
	brne	scdn1
	cpi	YH, 0x01			; at the end of disp RAM? 00
	brne	scdn1				; no, do again
	ldi	inpt, 0x20			; space
scdn2:
	st	-Z, inpt			; fill last line with spaces
	cpi	ZL, 0x00			; 60
	brne	scdn2
	cpi	ZH, 0x01			; at the end of disp RAM? 00
	brne	scdn2				; no, do again
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

SCROLLLF:
	push	YL
	push	YH
	ldi	YL, 0x00			; start of first line 60
	ldi	YH, 0x01			; 00
	ldi	ZL, 0x01			; start of second line 61
	ldi	ZH, 0x01			; 00
sclf1:
	ld	inpt, Z+			; get line 2 
	st	Y+, inpt			; place in line 1
	cpi	ZL, 0xD0			; 30
	brne	sclf1
	cpi	ZH, 0x08			; at the end of disp RAM?
	brne	sclf1				; no, do again
	ldi	inpt, 0x20			; space
sclf2:
	st	Y, inpt				; fill last line with spaces
	sbiw	YL, 0x20			; 
	sbiw	YL, 0x30			; 
	cpi	YL, 0xFF			; 5F
	brne	sclf2				;
	cpi	YH, 0x01			; at the end of disp RAM? 00
	brne	sclf2				; no, do again
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

SCROLLRT:
	push	YL
	push	YH
	ldi	YL, 0xCF			; end of line 24  2F
	ldi	YH, 0x08			; 
	ldi	ZL, 0xD0			; end of line 25  30
	ldi	ZH, 0x08			; 
scrt1:
	ld	inpt, -Y			; get line 2 
	st	-Z, inpt			; place in line 1
	cpi	YL, 0x00			; 60
	brne	scrt1
	cpi	YH, 0x01			; at the end of disp RAM? 00
	brne	scrt1				; no, do again
	ldi	inpt, 0x20			; space
scrt2:
	st	Y, inpt				; fill left column with spaces
	adiw	YL, 0x30			; 
	adiw	YL, 0x20			; 
	cpi	YL, 0xD0			; 30
	brne	scrt2
	cpi	YH, 0x08			; at the end of disp RAM?
	brne	scrt2				; no, do again
	pop	YH
	pop	YL
	ld	curschr, Y			; save chr under cursor
	ret

Curs_us:
	ldi	Cursctl, 0x5F			; no, set cursor to underscore
	ret

Curs_blk:
	ldi	Cursctl, 0x0D			; set cursor to BLOCK
	ret

Curs_off:
	ldi	Cursctl, 0x00			;
	ret

Curs_blnk:
	ldi	Cursclk, 0x00
	ret

Curs_on:
	ldi	Cursclk, 0xff
	ret

Cursup:
	cpi	vpos, 0x00			; 
	breq	Null				; already at top, nothing to do
	dec	vpos				;
	sbiw	YL, 0x20			; 
	sbiw	YL, 0x30			; 
	ld	curschr, Y			; save chr under cursor
	ret

CursDn:
	cpi	vpos, 0x18			; 
	breq	Null				; already at bottom, nothing to do
	inc	vpos				;
	adiw	YL, 0x20			; 
	adiw	YL, 0x30			; 
	ld	curschr, Y			; save chr under cursor
	ret

Curslf:
	cpi	hpos, 0x00			; 
	breq	Null				; already at top, nothing to do
	dec	hpos				;
	sbiw	YL, 0x01			; 
	ld	curschr, Y			; save chr under cursor
	ret

Cursrt:
	cpi	hpos, 0x4F			; 
	breq	Null				; already at bottom, nothing to do
	inc	hpos				;
	adiw	YL, 0x01			; 
	ld	curschr, Y			; save chr under cursor
	ret

; Some sequences have a command byte followed by an argument - eg cursor horiz/vert position 
; For these, store the command byte.
; Then, when the second byte appears, ProcChr determines that there is a command
; waiting for a parameter so it then calls the appropriate method. "inpt" would then
; contain the second byte parameter.
Command:
	mov 	cmdVal,inpt
	ret

NULL:
	ret

.cseg
.org 0x500
;
; jump table for each chr input from host
;

	rjmp	unknown				; 0
	rjmp	unknown				; 1
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	backspace			; 8
	rjmp	tab				; 9
	rjmp	linefeed			; A
	rjmp	null				; B
	rjmp	formfeed			; C = "clear screen"
	rjmp	return				; D
	rjmp	unknown				; E
	rjmp	unknown				; F
	rjmp	unknown				; 10
	rjmp	unknown				; 1
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	unknown				; A
	rjmp	unknown				; B
	rjmp	unknown				; C
	rjmp	unknown				; D
	rjmp	CursHome			; E	mod. 2013.6.18
	rjmp	linefeed			; F	mod. 2013.6.18
	rjmp	ascii				; 20
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 30
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 40
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 50
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 60
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	ascii				; F
	rjmp	ascii				; 70
	rjmp	ascii				; 1
	rjmp	ascii				; 2
	rjmp	ascii				; 3
	rjmp	ascii				; 4
	rjmp	ascii				; 5
	rjmp	ascii				; 6
	rjmp	ascii				; 7
	rjmp	ascii				; 8
	rjmp	ascii				; 9
	rjmp	ascii				; A
	rjmp	ascii				; B
	rjmp	ascii				; C
	rjmp	ascii				; D
	rjmp	ascii				; E
	rjmp	delete				; 7F


	rjmp	special				; 80
	rjmp	special				; 1
	rjmp	special				; 2
	rjmp	special				; 3
	rjmp	special				; 4
	rjmp	special				; 5
	rjmp	special				; 6
	rjmp	special				; 7
	rjmp	special				; 8
	rjmp	special				; 9
	rjmp	special				; A
	rjmp	special				; B
	rjmp	special				; C
	rjmp	special				; D
	rjmp	special				; E
	rjmp	special				; F
	rjmp	special				; 90
	rjmp	special				; 1
	rjmp	special				; 2
	rjmp	special				; 3
	rjmp	special				; 4
	rjmp	special				; 5
	rjmp	special				; 6
	rjmp	special				; 7
	rjmp	special				; 8
	rjmp	special				; 9
	rjmp	special				; A
	rjmp	special				; B
	rjmp	special				; C
	rjmp	special				; D
	rjmp	special				; E
	rjmp	special				; F
	rjmp	unknown				; A0
	rjmp	unknown				; 1
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	unknown				; A
	rjmp	unknown				; B
	rjmp	unknown				; C
	rjmp	unknown				; D
	rjmp	unknown				; E
	rjmp	unknown				; F
	rjmp	unknown				; B0
	rjmp	unknown				; 1
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7 
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	CursHome			; A
	rjmp	Curs_off			; B cursor off
	rjmp	Curs_blk			; C cursor block
	rjmp	Curs_us				; D cursor underscore
	rjmp	Curs_blnk			; E cursor blinking
	rjmp	Curs_on				; F cursor solid
	rjmp	command				; C0 SetColumn command (next byte=col number)
	rjmp	command				; 1  SetRow command (next byte=row number)
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	unknown				; A
	rjmp	unknown				; B
	rjmp	unknown				; C
	rjmp	unknown				; D
	rjmp	unknown				; E
	rjmp	unknown				; F
	rjmp	del_sol				; D0
	rjmp	del_eol				; 1
	rjmp	del_sop				; 2
	rjmp	del_eop				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	unknown				; A
	rjmp	unknown				; B
	rjmp	unknown				; C
	rjmp	unknown				; D
	rjmp	unknown				; E
	rjmp	unknown				; F
	rjmp	unknown				; E0
	rjmp	unknown				; 1
	rjmp	unknown				; 2
	rjmp	unknown				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	CursUp				; 8
	rjmp	CursDn				; 9
	rjmp	CursLf				; A
	rjmp	CursRt				; B
	rjmp	scrollup			; C
	rjmp	scrolldn			; D
	rjmp	scrolllf			; E
	rjmp	scrollrt			; F
	rjmp	Set_Lower			; F0
	rjmp	Set_Upper			; 1
	rjmp	Set_prim			; 2
	rjmp	Set_alt				; 3
	rjmp	unknown				; 4
	rjmp	unknown				; 5
	rjmp	unknown				; 6
	rjmp	unknown				; 7
	rjmp	unknown				; 8
	rjmp	unknown				; 9
	rjmp	unknown				; A
	rjmp	unknown				; B
	rjmp	unknown				; C
	rjmp	unknown				; D
	rjmp	unknown				; E
	rjmp	unknown				; FF
