;/******************************************************************************/
;/  NTSC/PAL Video Display (40 character x 25 line monochrome text)		/
;/  8 bit parallel load with latch and busy handshake lines 			/
;/										/
;/  Designed and written by Daryl Rictor (c)2003-2004				/
;/******************************************************************************/

;******************************************************************************
; modify for ATMega328P  by neko Java 2013.01.18
;******************************************************************************
; include ATMega328P definitions & program specific definitions and macros
;******************************************************************************
.include  "m328Pdef.inc"
.include  "defs.inc"
;******************************************************************************
; Code Begins at 0x000
; System IRQ jump table from 0x0000 - 0x0012
;******************************************************************************
	jmp 	RESET 				; Reset Handler
	jmp 	EXT_INT0 			; IRQ0 Handler
	jmp 	EXT_INT1 			; IRQ1 Handler
	jmp 	PC_INT0 			; Pin Change IRQ0 Handler
	jmp 	PC_INT1 			; Pin Change IRQ1 Handler
	jmp 	PC_INT2 			; Pin Change IRQ2 Handler
	jmp 	WDT 				; WDT Handler
	jmp 	TIM2_COMPA 			; Timer2 Compare Handler
	jmp 	TIM2_COMPA 			; Timer2 Compare Handler
	jmp 	TIM2_OVF 			; Timer2 Overflow Handler
	jmp 	TIM1_CAPT 			; Timer1 Capture Handler
	jmp 	TIM1_COMPA 			; Timer1 CompareA Handler
	jmp 	TIM1_COMPB 			; Timer1 CompareB Handler
	jmp 	TIM1_OVF 			; Timer1 Overflow Handler
	jmp 	TIM0_COMPA 			; Timer1 CompareA Handler
	jmp 	TIM0_COMPB 			; Timer1 CompareA Handler
	jmp 	TIM0_OVF 			; Timer0 Overflow Handler
	jmp 	SPI_STC 			; SPI Transfer Complete Handler
	jmp 	USART_RXC 			; USART RX Complete Handler
	jmp 	USART_UDRE 			; UDR Empty Handler
	jmp 	USART_TXC	 		; USART TX Complete Handler
	jmp 	ADCC 				; ADC Conversion Complete Handler
	jmp 	EE_RDY	 			; EEPROM Ready Handler
	jmp 	ANA_COMP 			; Analog Comparator Handler
	jmp 	TWSI 				; Two-wire Serial Interface Handler
	jmp 	SPM_RDY 			; Store Program Memory Ready Handler
	
;******************************************************************************
; unused IRQ vector's point to software reset code
;******************************************************************************

EXT_INT0:
EXT_INT1:
PC_INT0:
PC_INT1:
PC_INT2:
WDT:
TIM2_COMPA:
TIM2_COMPB:
TIM2_OVF:
TIM1_CAPT:
;TIM1_COMPA:
TIM1_COMPB:
TIM1_OVF:
TIM0_COMPA:
TIM0_COMPB:
TIM0_OVF:
SPI_STC:
USART_RXC:
USART_UDRE:
USART_TXC:
ADCC:
EE_RDY:
ANA_COMP:
TWSI:
SPM_RDY:

;******************************************************************************
; Program Starts here on Reset
;******************************************************************************
RESET:					; first line executed after reset 
	CLI				; disable interupts

	ldi	J,0x00
	sts	UCSR0B, J
	mov	cmdVal, J

	LDI	J, HIGH(RAMEND)		;stack
	OUT	SPH, J
	LDI	J, LOW(RAMEND)
	OUT	SPL, J

;******************************************************************************
; Initialize the I/O Ports
;******************************************************************************
; port D pins (video data bus)
	ldi	J, 0x00			; data 0
	out	portd, J		; set port D outputs to 0
	LDI	J, 0xFF			; pins 0 - 7 of port D
	out	ddrd, J			; set PORTD pins to outputs

; port B,C pins (control, handshake and host data input bus(PB0-3,PC0-3))
	LDI	J, 0x00			; 
	out	portb, J		; disable pullup's
	out	portc, J		; disable pullup's
	SBI	portb, syncpin		; turn on sync bit (done after mode select below)
	SBI	portb, ldsrpin		; turn shift load bit on 
	CBI	portc, ackdpin		; set ack pin to low
	LDI	J, 0x20			; ackdpin(PC5) of port C = output
	out	ddrc, J			; 
	LDI	J, 0x30			; syncpin(PB5), ldsrpin(PB4) of port B = output
	out	ddrb, J			;

; Load Shift Register with all 0' so display remains blank
	CBI	portb, ldsrpin		; set SR to LOAD mode
	SBI	portb, ldsrpin		; set SR to SHIFT mode

;******************************************************************************
; NTSC System
;******************************************************************************

SETNTSC:				; no, set NTSC mode
	ldi	J, 0x06			; 0x0106 Last vertical line 262
	mov	lastline, J		;
	ldi	J, 0x28			; first vertical line of active disp
	mov	line1, J		;
	ldi	J, 0x03			; 0x03f8 = 1016 clocks per line (63.5us)
	sts     OCR1AH, J        
	ldi	J, 0xf8			; 0x03f8 = 1016 clocks per line (63.5us)
	sts     OCR1AL, J		; set Counter 1 to cycle from 0 to 1016 (1024 PAL) and IRQ

RESTORE_C4:
	SBI	portc, ackdpin		; raise PC5 to hi level

;******************************************************************************
; Initialize Timer 1 
;******************************************************************************
;   OCR1A will hold the value of 1016/1024 and be used to reset the counter and 
;        cause an IRQ that will start the HSYNC process
;   The counter will count each clock cycle (16MHz - no prescaling)

	ldi     J, 0x00
	sts     TCCR1A, J
	ldi     J, 0x09			; set timer 1 to CTC mode 4
	sts     TCCR1B, J		

	ldi     J, 0x02			; allow OCR1A IRQ's
	sts     TIMSK1, J		

;******************************************************************************
; Init other CPU I/O registers
;******************************************************************************
	ldi     J, 0x80
	out     ACSR, J			; turn off comparator

	;ldi     J, 0x80
	;out	MCUCR, J		; enable sleep mode

	;ldi	J, 0x08			; set stack pointer to top of SRAM 
	;out	SPH, J			; (24 bytes max stack size)
	;ldi	J, 0x5f			; uses 6 bytes (IRQA-2, IRQB-2, ProcChr sub-2
	;out	SPL, J			;

;******************************************************************************
; Initialize the Display SRAM
;******************************************************************************
; fill display SRAM with the Startup Banner 
	ldi	ZL, 0x00
	ldi	ZH, 0x1C		; set to start of Banner in Prog Mem 38
	ldi	YL, 0x00
	ldi	YH, 0x01		; set to start of SRAM
	ldi	XL, 0xD0
	ldi	XH, 0x07		; set X to 2000
	ldi	J, 0x20			; 
frloop:
	lpm	J, Z+			; get boot image
	st 	Y+, J			; save to SRAM
	sbiw	XL, 1			; dec X
	brne	frloop			; do until X=0

;******************************************************************************
; Initialize Program Variables
;******************************************************************************
	ldi	chrln, 0x00		; keeps track of font gen line (0-7)
	ldi	vlineh, 0x00		; vertical line counter 0-262
	ldi	vline, 0x00		;  ""
	ldi	YL, 0x00		; 60
	ldi	YH, 0x01		; init y pointer 00
	ldi	hpos, 0x00		; keeps track of horizontal chr pos (0-39)
	ldi	vpos, 0x00		; keeps track of vertical chr pos (0-24)
	ldi	Cursclk, 0x00		; init cursor blink timer
	ld	Curschr, Y		; init cursor character
	ldi	Cursctl, 0x5F		; init cursor '_' character
	ldi	J, 0x00			; 0
	mov	fnthl, J		; set high/low flag to low
	mov	fntsel, J		; set Font to primary

;******************************************************************************
;  Reset Timer 1 and clear Busy Status Flag
;******************************************************************************
	ldi     J, 0x00			; clear timer 1
	sts     TCNT1H, J		; 
	sts     TCNT1L, J		; 

	ldi	J, 0x02			; clear timer 1 IRQ's
	out	TIFR1, J		; clear timer 1 IRQ's

	CBI	portc, ackdpin		; clear the ACK line to the SBC 
	SBI	portc, ackdpin		; 

	ldi	CursClk, 0xFF		 

	ldi 	inpt,0x0C		; CLS
	rcall	ProcChr
	ldi 	inpt,0xBE		; Cursor blink
	rcall	ProcChr

	SEI				; Ready to go - enable system IRQ's

		
;******************************************************************************
; Main Loop 
;******************************************************************************
Main:
	cpi	Cursctl, 0x00		; is the cursor turned on?
	breq	main2			; no, skip control code
	cpi	Cursclk, 0xff		; is clk at 0xFF?
	breq	main1			; yes - solid cursor (no blink)
	cpi	Cursclk, 0x1C		; is it above 28 frames?
	brpl	main2			; if >25, show Cursor
	st	Y, Curschr		; if clr, show character
	rjmp	main3			; 
main1:
	st	Y, Cursctl		; if set show cursor
	rjmp	main3
main2:
	st	Y, Cursctl		; if set show cursor
	cpi	Cursclk, 0x37		; is it >56 
	brmi	main3			; no
	clr	Cursclk			; reset timer
main3:

	; If the ack and data rdy are the same state then there is no
	; data waiting
	in	J,PINC
	andi	J,0x10			; data abalaible(PC4)
	in	K,PORTC
	andi	K,0x20			; ack(PC5)
	lsr	K
	eor	J,K
	breq	Main

	st	Y, Curschr		; restore chr under cursor

	in 	J, pinc
	in	K, pinb
	andi	J, 0x0F
	lsl	K
	lsl	K
	lsl	K
	lsl	K
	or	J, K
	mov	inpt, J			; save data

	; flip the ack bit
	in	K,PORTC
	ldi	J,0x20			; ack(PC5)
	eor	K,J
	out	PORTC,K

	rcall	ProcChr			; process the host data
	rjmp	Main			; repeat

;******************************************************************************
; IRQ Service Routines
;******************************************************************************

TIM1_COMPA:				; IRQ service for OCR1A 
.include "vidgen.inc"			; file that holds the video generation code
	reti				; done

;******************************************************************************
; include subroutines and Constant Data tables
;******************************************************************************
.include "ProcChr.inc"			; terminal emulation subroutines 0x500-0x5FF
.include "banner.inc"			; opening screen banner 0x600-0x7F3
.include "primfont.inc"			; disp PRIMARY font 0x800-0xBFF
.include "altfont.inc"			; disp ALT font 0xC00-0xFFF

;******************************************************************************
; End of Program
;******************************************************************************
